'''  PROPDOS Propeller MiniDOS V1.7b
'''  By Jeff Ledger / AKA Oldbitcollector
{
 See Official PropDOS website for Full Instructions: http://www.orrtech.net/propdos
 Many people have contributed to PropDOS: Type HELP to display COMMANDS & CREDITS.
 
 Perfer a different FONT?     Change font in OBJ section of "AIGeneric_Driver_002" 

 PropDOS COMMANDS:
 -----------------
 DIR = show files on the actual root or Virtual Folder/Directory
    [ F1 and % will also excute this command.]
 DIR/W = show actual files on SD (wide display)
    [ F4 and $ will also execute this command.]

 DIR *.ext = show files with .EXT
 DEL = delete file (del filename)
 SPIN = load/run binary from SD (spin filename)
    [ .BIN files can be executed 'DOS-like' by simply typing the ]
    [ filename without the .bin extension.                       ]

 CLS = clear the screen
 TYPE = display a text file (type filename)
 HELP = display HELP screen
 PLAY = load/play an hmus file (play filename)
 STOP = stop playing current hmus file
 REBOOT = Reboot the Propeller
 COPY CON filename = Simple text file creator, exit with Control-Z on a blank line
 WPLAY = play 16k mono wav file (wplay filename)
    [ Includes both versions of Raymans wav player for backward compatibility ]
    [ Should detect original 16k wav files and play them properly.]

 LABEL <filename> = Assign Filename to a Virtual Folder/Directory
 CD <directory> = Move to Virtual Folder/Directory  {omit .DIR in directory name}
 CD\ = Move back to the either Virtual Root or Acutal Root
    [Default settings are for actual SD root, can be changed below.]

}
DAT
'' Specify alternate binary to run if [ESC] is held down at boot
ESC     byte "menu.bin"

'' Specify Virtual root directory/label or 0 for real root.
'ROOT    byte "0",0              '' System default, displays actual files at ROOT.
ROOT    byte "root",0           '' Must be created using LABEL command with 1 filename.
 
CON
  '' I/O SETTINGS

   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000
  
  '' Adjust for typical location of the keyboard. Usually 26 on the demoboard
  
   keyboard  = 26
   video     = 12
   
  '' Adjust for location of the SD card.  Typical settings below.
{
  '' Typical Hydra Configuration: 0-7 Left for NES/Audio Connections
  
   spiDO     = 16
   spiClk    = 17
   spiDI     = 18
   spiCS     = 19
}

  '' Typical Demoboard/Protoboard Configuration
  
   spiDO     = 0
   spiClk    = 1
   spiDI     = 2
   spiCS     = 3


'' Change these colors to change DIR display colors

  wavfiles    = lgreen           'Display color of .WAV files in DIR
  hmufiles    = skyblue          'Display color of .HMU files in DIR
  binfiles    = yellow           'Display color of .BIN files in DIR
  basfiles    = lpurple          'Display color of .BAS files in DIR
  dirfiles    = lred             'Display color of .DIR files in DIR  
  textcolor   = white            'Display color any other time.
  
DAT   ''**************************************************
      ''****    NO CHANGES REQUIRED PAST THIS POINT   ****
      ''**************************************************
CON
  '' Key Assignments 

  fReturn           = 13
  bspKey            = $C8
  escape            = $CB
  backspace         = 8
  CR                = $0D
  LF                = $0A
  TAB               = $09
  BKS               = $08


 '' Basic Color Settings

  BLACK             = $02 
  GREY              = $03
  LWHITE            = $06
  BWHITE            = $07
  LMAGENTA          = $A3 
  WHITE             = $AF
  RED               = $A2
  LRED              = $1A
  LPURPLE           = $1B
  PURPLE            = $03 
  LGREEN            = $1C
  SKYBLUE           = $1D
  YELLOW            = $1E
  BYELLOW           = $A6
  LGREY             = $1F
  DGREY             = $08
  GREEN             = $A4
  ORANGE            = $06
  BLUE              = $A1 
  CYAN              = $1C
  MAGENTA           = $FC
  LYELLOW           = $1E

  '' Wav Player Assignments
  
  buffSize          = 256
  
  HORIZONTAL_PIXELS =  80
  VERTICAL_PIXELS =  96
   
OBJ
     key   : "comboKeyboard"             ' Keyboard Driver
     text  : "aigeneric_driver"          ' AIGeneric Video Driver
     fsrw  : "fsrwFemto"                 ' Modified SD routines
     hss   : "yma_hss_v1.2"              ' Hydra Sound System
     tv    : "potatohead--hi-color-80x96"   'the TV driver
  capture  : "video_capture"
  process  : "video_process_80x96"
  sound    : "A_modulator"     
'       ir  : "ir_cog_fortec"
 
      
VAR
      byte     displayb[HORIZONTAL_PIXELS * VERTICAL_PIXELS]   'allocate display buffer in RAM

      byte command[32]
     byte f0[32]
     byte tbuf[20] ' SD Vars
     long ioControl[2] 
     long tp
     byte music[128]
     byte cline[42]
     byte ext[3]
     byte strbuff[5]

     byte folder[10]
     byte label[15]

     '' Wav Player Allocation
    long parameter1  'to pass @buff1 to ASM
    long parameter2  'to pass @buff2 to ASM
    long parameter3  'to pass sample rate to ASM
    long parameter4  'to pass #samples to ASM
    long buff1[buffSize]
    long buff2[buffSize]
    byte Header[44]

    long stack1[64]
    word MonoData[buffSize]
    long index,SyncPoint,ii
     
PUB mainProgram  | size, r,sta,bytes,counter,newcount,i,n,j,spare,t,Pass,Pos,keyval,tt,rr,SampleRate,Samples,hidden  

  hss.start
  key.start(keyboard)
  text.start(video) 
  fsrw.start(@ioControl)                         ' Start I2C/SPI driver
  text.color(textcolor) 
  '' Check for the required keyboard & SD card, gracefull abort if not found.
  if \fsrw.mount(spiDO,spiClk,spiDI,spiCS) < 0
     text.str(string(13,"No SD card present.",CR))
     text.str(string("SD card expected on: "))
     text.dec(spiDO)
     text.str(string(", "))
     text.dec(spiClk)
     text.str(string(", "))
     text.dec(spiDI)
     text.str(string(", "))
     text.dec(spiCS)
     text.str(string(13,CR,"System halted.",CR)) 
     abort 
  waitcnt(clkfreq + cnt)
  if not key.present                                
     text.str(string(13,"No keyboard present",CR))
     text.str(string("Keyboard expected on: "))
     text.dec(keyboard)
     text.str(string(13,CR,"System halted.",CR))
     abort

  waitcnt(clkfreq + cnt) 
  
  keyval:=key.key
      if keyval == 203
          fsrw.popen(@esc,"r")
          fsrw.bootSDCard


 
  text.color(LGREEN)
  text.str(string("Propeller VideoDOS V1.7b",CR,"By Jeff Ledger+Perry Mole",CR,CR))
  text.color(textcolor)

  bytefill(@folder,0,9)
  bytefill(@label,0,14)
  setroot
    
  repeat
   bytefill(@command,0,32) 'Clear the command buffer
   bytefill(@cline,0,14) ' Clear commandline buffer
   
   text.str(string("SD:\"))
   text.str(@folder)
   text.str(string(">"))
   
   input(@command,32)

   if textComp(@command,string("spin"),4) == true 
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
    fsrw.popen(@cline,"r")
    fsrw.bootSDCard
   
   elseif textComp(@command,string("dir/w"),5) or textComp(@command,string("$"),1) == true
      dirw

   elseif textComp(@command,string("dir *."),6) == true
    counter := 6 ' Starting point of command line (after space)
    newcount := 0
    repeat until newcount > 2
       cline[newcount] := command[counter]
       cline[newcount]-=32
       counter++
       newcount++
    text.out(13)
    text.str(string("Directory of SD:"))
    text.color(skyblue)
    text.str(string("*."))
    counter := 0
    repeat until counter > 4
      if cline[counter] <> 0
        text.out(cline[counter])
      counter++
    text.out(13)
    text.out(13)
    text.color(textcolor)
    fsrw.opendir
    repeat while fsrw.nextfile(@tbuf) == 0
      t:=StrRight(@tbuf,3)
      if strcomp(t,@cline)
         text.str(string(" "))
         text.str(@tbuf)
         text.out(13)
    text.out(13)

   elseif textComp(@command,string("dir"),3) or textComp(@command,string("%"),1) or textComp(@command,string("ls"),2) == true

     if folder[1]==0
       dosdir

     else
        hidden := true
        text.out(13)
        text.str(string("Virtual Directory:", 13))
        text.out(13)
        fsrw.opendir
        repeat while fsrw.nextfile(@tbuf) == 0     
           tt:=(StrRight(@tbuf,3))
           if textcomp(tt,string("DIR"),3) == true
             hidden:=false
           if hidden==false
             text.str(string(" "))
             text.color(dirfiles)
             text.str(@tbuf)
             text.color(textcolor)
             text.out(13)
             hidden:=true          
        counter:=0
        newcount:=0
        bytefill(@label,0,12)
        repeat until counter > 7
          if folder[counter]<>0
               newcount++
               label[counter]:=folder[counter]
          counter++ 
        label[newcount]:=46
        newcount++
        label[newcount]:=68
        newcount++
        label[newcount]:=73
        newcount++
        label[newcount]:=82
        fsrw.pclose

        r := fsrw.popen(@label,"r")  
        repeat
          r := fsrw.pgetc
          if r < 0
             text.out(13)
             text.color(white)
             quit
          if r > 0
             ''Convert lowercase to UPPERCASE
               if r > 96 and r < 123
                   r := r-=32
               text.color(cyan)
               text.out(r)  

   elseif textComp(@command,string("cd .."),5) or textComp(@command,string("cd.."),4) or textComp(@command,string("cd\"),3) == true
       setroot
       bytefill(@command,0,32)

   elseif textComp(@command,string("cd "),3) == true
     counter :=3
     newcount :=0
     repeat until newcount > 7
       folder[newcount]:=command[counter]
       counter++
       newcount++
     counter:=0

   elseif textComp(@command,string("label"),5)==true
     counter :=6
     newcount:=0
     repeat until newcount > 11
       cline[newcount]:=command[counter]
       counter++
       newcount++
     counter:=0
     text.str(string(13,"Define "))
     text.color(lgreen)
     repeat until counter == 12
        if cline[counter] <> 0
          text.out(cline[counter])
        counter++
     text.color(white)
     text.str(string(" label:"))
     counter:=0
     newcount:=0
     bytefill(@command,0,32) 'Clear the command buffer  
     input(@command,8)
     repeat until counter > 7
       if command[counter]<>0
         newcount++
         label[counter]:=command[counter]
       counter++ 
     label[newcount]:=46
     newcount++
     label[newcount]:=68
     newcount++
     label[newcount]:=73
     newcount++
     label[newcount]:=82
      counter:=0
     r := fsrw.popen(@label,"a")
     r:=(32)'Add space before name to line up with .DIR
     fsrw.out(r)
      repeat
        r := (cline[counter]) 
         if cline[counter] <> 0
            fsrw.out(r)
         counter++
         if counter >11
            fsrw.out(13)
            text.out(13)
             quit
       
       
   elseif textComp(@command,string("reboot"),6) == true
     reboot

   elseif textComp(@command,string("del"),3) == true
     counter := 4 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     r := fsrw.popen(@cline,"d")
     text.str(string(13,"File deleted.",CR,CR)) 


   elseif textComp(@command,string("copy con"),4) == true 
     counter := 9 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
     r := fsrw.popen(@cline,"w")
     repeat
      bytefill(@cline,0,40) 'Clear the buffer 
      input(@cline,39)
        if textComp(@cline,string("z"),1) == true
          quit        
      counter := 0
      repeat 
         r := (cline[counter]) 
           if cline[counter] <> 0
             fsrw.out(r)

         counter++ 
         if counter == 39
           fsrw.out(13)
           quit

   
   elseif textComp(@command,string("play"),4) == true 
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     r := fsrw.popen(@cline,"r")
      repeat
        r := fsrw.pgetc
          if r < 0 or counter > 7999
            'play file
             hss.hmus_load(@music)'load Hmus file into player.
             hss.hmus_play 'Play loaded file.
             text.out(13)
             text.str(string("<"))
             text.color(blue)
             text.str(string("**"))
             text.color(textcolor)
             text.str(string("> Playing: "))
             text.color(purple)
             counter := 0
             repeat until counter > 14
              if cline[counter] <> 0
               text.out(cline[counter])
              counter++
             text.color(textcolor)
             text.str(string(" <"))
             text.color(blue)
             text.str(string("**"))
             text.color(textcolor)
             text.str(string(">",CR))
             text.color(skyblue)
             text.str(string("using Hydra Sound System v1.2",CR))
             text.color(yellow)
             text.str(string("By Andrew Arsenault",CR,CR))
             text.color(textcolor)
             quit
        music[counter] := r
        counter++

   elseif textComp(@command,string("wplay"),5) == true
     counter := 6 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     hss.stop
     i:=fsrw.popen(@cline, "r")
     i:=fsrw.pread(@Header, 44) 'read data words to input stereo buffer
     text.out(13)
     text.str(string("<"))
     text.color(blue)
     text.str(string("**"))
     text.color(textcolor)
     text.str(string("> Playing: "))
     text.color(purple)
     counter :=0
     repeat until counter > 14
      if cline[counter] <> 0
       text.out(cline[counter])
      counter++
     text.color(textcolor)
     text.str(string(" <"))
     text.color(blue)
     text.str(string("**"))
     text.color(textcolor)
     text.str(string(">",CR))


     'Get sample rate from header
     SampleRate:=Header[27]<<24+Header[26]<<16+Header[25]<<8+Header[24]
     ''text.dec(SampleRate)
     ''text.out(13)

     'get # samples from header
     Samples:=Header[43]<<24+Header[42]<<16+Header[41]<<8+Header[40]
     Samples:=Samples>>2

     '' Use Rays older wav player for backward compatibility.
     if SampleRate==16000
        text.color(skyblue)
        text.str(string("using 16kWavPlay",CR))
        text.color(yellow)
        text.str(string("By Raymond Allen",CR,CR))
        text.color(textcolor)
        bytefill(@buff1,0,buffSize)
        bytefill(@buff2,0,buffSize)
        bytefill(@stack1,0,100)
       
        COGNEW(Player(10, 11),@stack1)    
        n:=buffSize-1
        j:=buffsize*2
        repeat while j > 0 '(j==buffsize*2)  'repeat until end of file
       
           j:=fsrw.pread(@displayb, 80*96)   ' get video frame

           if (buff1[n]==0)
            j:=fsrw.pread(@MonoData, buffSize*2) 'read data words to input stereo buffer   
            'fill table 1
            repeat i from 0 to n
               buff1[i]:=($8000+MonoData[i])<<16
          if (buff2[n]==0)
            j:=fsrw.pread(@MonoData, buffSize*2) 'read data words to input stereo buffer  
            'fill table 1
            repeat i from 0 to n
               buff2[i]:=($8000+MonoData[i])<<16     

        fsrw.pclose
        hss.start

     '' Use Rays new wav player for 16-bit PCM WAV filess
     else
       'Start ASM player in a new cog
       text.color(skyblue)
       text.str(string("using PlayAsmWavB",CR))
       text.color(yellow)
       text.str(string("By Raymond Allen",CR,CR))
       text.color(textcolor)
       parameter1:=@buff1[0]
       parameter2:=@buff2[0]
       parameter3:=CLKFREQ/SampleRate  '#clocks between samples'1814'for 44100ksps,  5000 'for 16ksps
       parameter4:=Samples
       COGNEW(@ASMWAV,@parameter1)
       ' stop text video
       text.Close

       'Keep filling buffers until end of file
       ' note:  using alternating buffers to keep data always at the ready...
       n:=buffSize-1
       j:=buffsize*4   'number of bytes to read
       repeat while (j==buffsize*4)  'repeat until end of file
          
          if (buff1[n]==0)
            j:=fsrw.pread(@buff1, buffSize*4) 'read data words to input stereo buffer   

          if (buff2[n]==0)
            j:=fsrw.pread(@buff2, buffSize*4) 'read data words to input stereo buffer  

       fsrw.pclose
       hss.start
   
   elseif textComp(@command,string("vplay"),5) == true
     counter := 6 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++      
     counter := 0
     hss.stop
     i:=fsrw.popen(@cline, "r")
'     i:=fsrw.pread(@Header, 44) 'read data words to input stereo buffer
'     text.out(13)
'     text.str(string("<"))
'     text.color(blue)
'     text.str(string("**"))
'     text.color(textcolor)
'     text.str(string("> Playing: "))
'     text.color(purple)
'     counter :=0
'     repeat until counter > 14
'      if cline[counter] <> 0
'       text.out(cline[counter])
'      counter++
'     text.color(textcolor)
'     text.str(string(" <"))
'     text.color(blue)
'     text.str(string("**"))
'     text.color(textcolor)
'     text.str(string(">",CR))

    'Start ASM player in a new cog
'     text.color(skyblue)
'     text.str(string("using PlayAsmWavB",CR))
'     text.color(yellow)
'     text.str(string("By Raymond Allen",CR,CR))
'     text.color(textcolor)

       ' stop text video
     text.Close
       
     parameter1:=@buff1[0]
     parameter2:=@buff1[128] '2[0]
     parameter3:=5550 'CLKFREQ/SampleRate  '#clocks between samples'1814'for 44100ksps,  5000 'for 16ksps
     parameter4:=1024 'Samples
     
'     j := COGNEW(@ASMWAV,@parameter1)  
   'start the audio player in its own cog
     j := COGNEW(Player(10, 11),@stack1) 

       ' start graphic video
     repeat index from 0 to (HORIZONTAL_PIXELS * VERTICAL_PIXELS) 
        byte [@displayb] [index] := 3 '@@clut[1]  'There is a CLUT in this program though
           
     tv.start(@displayb)  ' Cog #1 start the tv cog & pass it the address of the bitmap 


'     hss.start    
       'Keep filling buffers until end of file
       ' note:  using alternating buffers to keep data always at the ready...
     n:=buffSize-1
     j:=buffsize*4   'number of bytes to read
     SyncPoint := cnt
     repeat while (j==buffsize*4)  'repeat until end of file
          
'          if (buff1[n]==0)
          j:=fsrw.pread(@buff1, 1024) 'buffSize*4) 'read data words to input stereo buffer   
          ii := 0
'          if (buff2[n]==0)
'            j:=fsrw.pread(@buff2, buffSize*4) 'read data words to input stereo buffer


             
          i:=fsrw.pread(@displayb, 80*96)  ' get video  frame

          waitcnt(SyncPoint += 5_333_333) ' 15 frames per second
            
     fsrw.pclose
     reboot
        
   elseif textComp(@command,string("cls"),3) == true
     text.out($00) 

   elseif textComp(@command,string("stop"),4) == true
     hss.hmus_stop
     text.out(13)

   elseif textComp(@command,string("type"),4) == true
     counter := 5 ' Starting point of command line (after space)
     newcount := 0
     repeat until newcount > 14
       cline[newcount] := command[counter]
       counter++
       newcount++
     r := fsrw.popen(@cline,"r")  
      repeat
        r := fsrw.pgetc
         if r < 0
           text.out(13)
           quit
        if r > 0
          text.out(r)

   elseif command[1] <> 0    '' AUTO-EXECUTE A .BIN FILE LIKE DOS
      counter := 0
      repeat until counter > 13
         if command[counter] > 0
            '' Convert Lowercase to UPPERCASE
            if command[counter] > 96 and command[counter] < 123
               command[counter] := command[counter]-=32
            cline[counter] := command[counter]
         counter++ 

      counter:=strsize(@cline)
      counter--
      counter++                   '' AAAAAARRRRRRGGG!!!
      cline[counter]:=46          '' This paticular routine
      counter++                   '' proved to be a bit challenging.
      cline[counter]:=66          '' To allow .BIN's to execute,
      counter++                   '' while ignoring everything else.
      cline[counter]:=73          '' --works perfectly.. :)
      counter++
      cline[counter]:=78
      rr:=strsize(@cline)
      fsrw.opendir
      repeat while fsrw.nextfile(@tbuf) == 0
        tt:=(StrRight(@tbuf,3))
        if textcomp(tt,string("BIN"),3) == true
           if textComp(@cline,@tbuf,rr) == true
              fsrw.popen(@cline,"r")
              fsrw.bootSDCard

PUB Player(PinR, PinL) | n,i,nextCnt,rate,dcnt
  'this should only play audio if there is audio to play. i need some kind of flag that can
  'be read to tell this function which if any buffer is full, and then this function
  'needs to set that flag to DONE

  'Set pins to output mode
  DIRA[PinR]~~                              'Set Right Pin to output
  DIRA[PinL]~~                              'Set Left Pin to output

  'Set up the counters
  CTRA:= %00110 << 26 + 0<<9 + PinR         'NCO/PWM Single-Ended APIN=Pin (BPIN=0 always 0)
  CTRB:= %00110 << 26 + 0<<9 + PinL         'NCO/PWM Single-Ended APIN=Pin (BPIN=0 always 0)   

 
  i:=0
  NextCnt:=cnt+15000

 repeat
      repeat ii from 0 to 1023
         NextCnt+=5550 'should be 10000 for 8 ksps
         waitcnt(NextCnt)
         FRQA:=(byte[@buff1+ii])<<24
         FRQB:=FRQA     
'         rate := byte[@mywavdata+i]~
        
PUB dirw | r, sta, bytes,b,d,tt,hidden,counter,keystroke

   text.out(13)
   text.str(string("Directory of SD", 13))
   text.out(13)
 
   fsrw.opendir                            
   b := 0
   hidden:= false                                  
   d := false                              
   repeat while fsrw.nextfile(@tbuf) == 0
      tt:=(StrRight(@tbuf,3))
      if textcomp(tt,string("HID"),3) == true
         hidden:=true
      if textcomp(tt,string("WAV"),3) == true
         text.color(wavfiles)
      if textcomp(tt,string("BIN"),3) == true
         text.color(binfiles)
      if textcomp(tt,string("HMU"),3) == true
         text.color(hmufiles)
      if textcomp(tt,string("BAS"),3) == true
         text.color(basfiles)
      if textcomp(tt,string("DIR"),3) == true
         text.color(dirfiles)             
      if hidden == false
        d := true                            
        if b == 39                         '' Grepped from FemtoBASIC
           text.out(13)                    '' Nice Wide Directory Listing!
           b := 0                                 
        if counter > 72
           counter:=0
           keystroke:=0
           text.color(purple)
           text.str(string("PRESS A KEY or [ESC]:"))           
           repeat until keystroke <> 0             
             keystroke := key.key
           text.str(string(BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS,BKS))
           text.color(textcolor)  
           if keystroke == 203
             quit
        counter++     
        text.str(@tbuf)
        text.color(textcolor)                     
        repeat 13 - strsize(@tbuf)             
           text.out(" ")                      
        b := b + 13
      if hidden == true
         hidden := false                           
   if d                                    
      text.out(13)                      
      text.out(13)

PRI dosdir | r, sta, bytes,tt,hidden
   hidden := false
   text.out(13)
   text.str(string("Directory of SD", 13))
   text.out(13)
   fsrw.opendir
   repeat
     if fsrw.nextfile(@tbuf) == -1
      text.color(textcolor)
      text.out(13)
      quit
    tt:=(StrRight(@tbuf,3))
    if textcomp(tt,string("HID"),3) == true
         hidden:=true
    if textcomp(tt,string("WAV"),3) == true
         text.color(wavfiles)
    if textcomp(tt,string("BIN"),3) == true
         text.color(binfiles)
    if textcomp(tt,string("HMU"),3) == true
         text.color(hmufiles)
    if textcomp(tt,string("BAS"),3) == true
         text.color(basfiles)
    if textcomp(tt,string("DIR"),3) == true
         text.color(dirfiles)              
    if hidden==false
      text.str(string(" "))
      text.str(@tbuf)
      text.color(textcolor)
      text.out(13)
    if hidden==true
       hidden:=false
          
PUB input(data, size) | keystroke,len
   keystroke:=0
   len:=0
   text.out(02)
   repeat' until keystroke == 13 'Exit when Carriage Return is pressed
     keystroke := key.key

      if keystroke == 211 'F4 will show actual wide directory
         keystroke:=36

      if keystroke == 208 'F1 will show acutal dir or virtual dir
         keystroke:=37
         
      if keystroke == $C8       ' Check for Backspace Key
        if len > 0
         byte[data--] := 0
         size++
         len--
         text.out(8)
         text.out(32) 'These lines erase the cursor on backspace
         text.out(8)
         
         text.out(8)
         text.out(32)
         text.out(8)
         text.out(02)
         keystroke := 0

      if keystroke == $C8        'Clean up backspace if all is cleared
         keystroke:= 0

      if keystroke == 13
         text.out(8)
         text.out(32)
         text.out(8)
         text.out(13)
         quit
         
      if keystroke > 10 
        if size > 1
          text.out(8)
          text.out(keystroke)
          text.out(02)
          
          ''text.dec(keystroke) 'for troubleshooting
          if keystroke <> 13 
            byte[data++] := keystroke
            size--
            len++

          if keystroke==36
             text.out(8)
             text.out(8)
             text.str(string("dir/w"))
             text.out(13)
             quit
             
          if keystroke==37
             text.out(8)
             text.out(8)
             text.str(string("dir"))
             text.out(13)
             quit

         if keystroke==63
             text.out(8)
             text.out(8)
             text.str(string("help"))
             text.out(13)
             quit    
PRI textComp(str1,str2,len)
   repeat len
      if byte[str1++] <> byte[str2++]
         return false
   return true

PUB StrRight(str,len) | i,l,z,j

  l:=strSize(str)
  z:=0
  repeat i from l-len to l
    strbuff[z] := byte[@tbuf][i]
    z++
  return @strbuff

PUB setroot | newcount,counter

   if root[0]=="0"
      bytefill(@folder,0,8) ' Clear commandline buffer

   else
    bytefill(@folder,0,8)
     newcount :=0
     counter:=0
     repeat until newcount > 7
       folder[newcount]:=root[counter]
       counter++
       newcount++
     counter:=0
       
{PUB Player(PinR, PinL)|n,i,nextCnt,j

  '  SPIN 16-bit WAV Player Ver. 1
  '  Copyright 2007 Raymond Allen

  'Play the wav data using counter modules
  'although just mono, using both counters to play the same thing on both left and right pins

  'Set pins to output mode
  DIRA[PinR]~~                              'Set Right Pin to output
  DIRA[PinL]~~                              'Set Left Pin to output

  'Set up the counters
  CTRA:= %00110 << 26 + 0<<9 + PinR         'NCO/PWM Single-Ended APIN=Pin (BPIN=0 always 0)
  CTRB:= %00110 << 26 + 0<<9 + PinL         'NCO/PWM Single-Ended APIN=Pin (BPIN=0 always 0)   

  'Get ready for fast loop  
  n--
  i:=0
  j:=true
  NextCnt:=cnt+1005000

    'Play loop
    'This loop updates the counter with the new desired output level
    'Alternates between buff1 and buff2 so main program can keep buffers full
    repeat 
      repeat i from 0 to buffSize-1
        NextCnt+=5000   ' need this to be 5000 for 16KSPS   @ 80 MHz
        waitcnt(NextCnt)
        if (j)
          FRQA:=buff1[i]~
          FRQB:=FRQA
        else
          FRQA:=buff2[i]~
          FRQB:=FRQA  
      NOT j

}
DAT
  ORG 0
ASMWAV
'load input parameters from hub to cog given address in par
        movd    :par,#pData1             
        mov     x,par
        mov     y,#4  'input 4 parameters
:par    rdlong  0,x
        add     :par,dlsb
        add     x,#4
        djnz    y,#:par

setup
        'setup output pins
        MOV DMaskR,#1
        ROL DMaskR,OPinR
        OR DIRA, DMaskR
        MOV DMaskL,#1
        ROL DMaskL,OPinL
        OR DIRA, DMaskL
        'setup counters
        OR CountModeR,OPinR
        MOV CTRA,CountModeR
        OR CountModeL,OPinL
        MOV CTRB,CountModeL
        'Wait for SPIN to fill table
        MOV WaitCount, CNT
        ADD WaitCount,BigWait
        WAITCNT WaitCount,#0
        'setup loop table
        MOV LoopCount,SizeBuff  
        'ROR LoopCount,#1    'for stereo
        MOV pData,pData1
        MOV nTable,#1
        'setup loop counter
        MOV WaitCount, CNT
        ADD WaitCount,dRate


MainLoop
        SUB nSamples,#1
        CMP nSamples,#0 wz
        IF_Z JMP #Done
        waitcnt WaitCount,dRate

        RDLONG Right,pData
        ADD Right,twos      'Going to cheat a bit with the LSBs here...  Probably shoud fix this!    
        MOV FRQA,Right
        ROL Right,#16       '16 LSBs are left channel...
        MOV FRQB,Right
        WRLONG Zero,pData
        ADD pData,#4

        'loop
        DJNZ LoopCount,#MainLoop
        
        MOV LoopCount,SizeBuff        
        'switch table       ?
        CMP nTable,#1 wz
        IF_Z JMP #SwitchToTable2
SwitchToTable1
        MOV nTable,#1
        MOV pData,pData1
        JMP #MainLoop
SwitchToTable2
        MOV nTable,#2
        MOV pData,pData2
        JMP #MainLoop
        
                
Done
         'now stop
        COGID thisCog
        COGSTOP thisCog          

'Working variables
thisCog long 0
x       long 0
y       long 0
dlsb    long    1 << 9
BigWait long 100000
twos    long $8000_8000
        
'Loop parameters
nTable  long 0
WaitCount long 0
pData   long 0
LoopCount long 0
SizeBuff long buffsize
'Left    long 0
Right   long 0
Zero    long 0          

'setup parameters
DMaskR  long 0 'right output mask
OPinR   long 10 'right channel output pin #                        '   <---------  Change Right pin# here !!!!!!!!!!!!!!    
DMaskL  long 0 'left output mask 
OPinL   long 11 'left channel output pin #                         '   <---------  Change Left pin# here !!!!!!!!!!!!!!    
CountModeR long %00011000_00000000_00000000_00000000
CountModeL long %00011000_00000000_00000000_00000000


'input parameters
pData1   long 0 'Address of first data table        
pData2   long 0 'Address of second data table
dRate    long 5000  'clocks between samples
nSamples long 2000  